from typing import List
from bip32 import HARDENED_INDEX
from coincurve import PublicKey
from sha3 import keccak_256 as _keccak_256


def keccak_256(b: bytes) -> bytes:
    h = _keccak_256()
    h.update(b)
    return h.digest()


def parse_path(s: str) -> List[int]:
    """
    m / purpose' / coin_type' / account' / change / address_index
    """
    args = s.split("/")
    res = []
    for arg in args:
        if arg == "m":
            continue
        elif arg.endswith("'") or arg.endswith("H") or arg.endswith("h"):
            res.append(int(arg[:-1]) + HARDENED_INDEX)
        else:
            res.append(int(arg))

    return res


def get_eth_addr(pk: bytes) -> str:
    if len(pk) != 64:
        pk = PublicKey(pk).format(False)[1:]
    return f"0x{keccak_256(pk)[-20:].hex()}"



from typing import Tuple, List, Iterable
from bip32 import BIP32, HARDENED_INDEX
from mnemonic import Mnemonic

from bip44.utils import parse_path

COIN_PATHS = {
    "BTC": (44 + HARDENED_INDEX, HARDENED_INDEX),
    "ETH": (44 + HARDENED_INDEX, 60 + HARDENED_INDEX),
}

_Mnemonic = Mnemonic("english")


class Wallet:
    def __init__(self, mnemonic: str):
        self._mnemonic = mnemonic
        self._seed = _Mnemonic.to_seed(mnemonic)
        self._bip32 = BIP32.from_seed(self._seed)

    def _derive_secret(self, path: Iterable[int]) -> bytes:
        return self._bip32.get_privkey_from_path(path)

    def _derive_public(self, path: Iterable[int]) -> bytes:
        return self._bip32.get_pubkey_from_path(path)

    def derive_secret_key(self, path: str) -> bytes:
        return self._derive_secret(parse_path(path))

    def derive_public_key(self, path: str) -> bytes:
        return self._derive_public(parse_path(path))

    def derive_account(
        self, coin: str, account: int = 0, change: int = 0, address_index: int = 0
    ) -> Tuple[bytes, bytes]:
        coin_path = COIN_PATHS[coin.upper()]
        account_path = (account + HARDENED_INDEX, change, address_index)
        path = coin_path + account_path
        return (self._derive_secret(path), self._derive_public(path))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bip44']

package_data = \
{'': ['*']}

install_requires = \
['bip32>=0.0.7,<0.0.8', 'mnemonic>=0.19,<0.20', 'pysha3>=1.0.2,<2.0.0']

setup_kwargs = {
    'name': 'bip44',
    'version': '0.0.1',
    'description': 'Simple python bip44 implementation. Mnemonic + bip32.',
    'long_description': '# python-bip44\n\nSimple python bip44 implementation. Mnemonic + bip32.\n\n## Install\n`pip install bip44`\n\n## Quick Start\n```python\n>>> from coincurve import PrivateKey\n>>> from bip44.wallet import Wallet\n>>> from bip44.utils import get_eth_addr\n>>> mnemonic = "purity tunnel grid error scout long fruit false embody caught skin gate"\n>>> w = Wallet(mnemonic)\n>>> sk, pk = w.derive_account("eth", account=0)\n>>> sk = PrivateKey(sk)\n>>> sk.public_key.format() == pk\nTrue\n>>> get_eth_addr(pk)\n\'0x7ad23d6ed9a1d98e240988bed0d78e8c81ec296c\'\n```\n',
    'author': 'Weiliang Li',
    'author_email': 'to.be.impressive@gmail.com',
    'maintainer': 'Weiliang Li',
    'maintainer_email': 'to.be.impressive@gmail.com',
    'url': 'https://github.com/kigawas/python-bip44',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
